@@ -17,6 +17,25 @@
    write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.  */
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+/* AIX requires this to be the first thing in the file.  */
+#ifndef __GNUC__
+# if HAVE_ALLOCA_H
+#  include <alloca.h>
+# else
+#  ifdef _AIX
+ #pragma alloca
+#  else
+#   ifndef alloca /* predefined by HP cc +Olibcalls */
+char *alloca ();
+#   endif
+#  endif
+# endif
+#endif
+
 #include "gnome-defs.h"
 #include "gnome-exec.h"
 #include "gnome-util.h"
@@ -121,6 +140,8 @@ gnome_execute_async_with_env_fds (const char *dir, int argc,
     break;
 
   default: /* parent process */
+    /* do nothing */
+    break;
   }
 
   close(comm_pipes[1]);
