@@ -35,7 +35,7 @@ typedef struct _Paper Paper;
 typedef struct _Unit Unit;
 
 GList*		gnome_paper_name_list	(void);
-const Paper	*gnome_paper_with_name	(const gchar *name);
+const Paper	*gnome_paper_with_name	(const gchar *papername);
 const Paper	*gnome_paper_with_size	(double pswidth, double psheight);
 
 const gchar	*gnome_paper_name	(const Paper *paper);
@@ -49,7 +49,7 @@ gdouble		gnome_paper_bmargin	(const Paper *paper);
 const gchar	*gnome_paper_name_default	(void);
 
 GList*		gnome_unit_name_list	(void);
-const Unit	*gnome_unit_with_name	(const gchar *name);
+const Unit	*gnome_unit_with_name	(const gchar *unitname);
 
 gdouble		gnome_paper_convert	(double psvalue, const Unit *unit);
 double          gnome_paper_convert_to_points (double othervalue, const Unit *unit);
