@@ -105,10 +105,13 @@ gnomelib_option_cb(poptContext ctx, enum poptCallbackReason reason,
 #endif
 		break;
 	default:
+		/* do nothing */
+		break;
 	}
 }
 
 static const struct poptOption gnomelib_options[] = {
+        { NULL, '\0', POPT_ARG_INTL_DOMAIN, PACKAGE, 0, NULL, NULL},
 	{ NULL, '\0', POPT_ARG_CALLBACK|POPT_CBFLAG_POST, gnomelib_option_cb, 0, NULL, NULL},
 	{ "disable-sound", '\0', POPT_ARG_NONE,
 	  &disable_sound, 0, N_("Disable sound server usage"), NULL},
@@ -134,27 +137,8 @@ gnomelib_init (const char *app_id,
 	gnome_app_id = (char *)app_id;
 	gnome_app_version = (char *)app_version;
 
-	gnome_user_home_dir = getenv ("HOME");
+	gnome_user_home_dir = gnome_util_user_home();
 
-	if (!gnome_user_home_dir){
-		char *user;
-		struct passwd *pw;
-		
-		user = getenv ("USER");
-		if (user)
-			pw = getpwnam (user);
-		else 
-			pw = getpwuid (getuid ());
-
-		if (pw)
-			gnome_user_home_dir = g_strdup (pw->pw_dir);
-
-		endpwent ();
-	}
-
-	if (gnome_user_home_dir == 0)
-		gnome_user_home_dir = "/";
-			
 	/*
 	 * never freed - gnome_config currently uses this, and it's better
 	 * to figure it out once than to repeatedly get it
