@@ -34,7 +34,7 @@
 static void gnome_app_class_init (GnomeAppClass *class);
 static void gnome_app_init       (GnomeApp      *app);
 static void gnome_app_destroy    (GtkObject     *object);
-static void gnome_app_realize    (GtkWidget *widget);
+static void gnome_app_show       (GtkWidget *widget);
 
 static gchar *read_layout_config  (GnomeApp *app);
 static void   write_layout_config (GnomeApp *app, GnomeDockLayout *layout);
@@ -112,7 +112,7 @@ gnome_app_class_init (GnomeAppClass *class)
 
 	object_class->destroy = gnome_app_destroy;
 
-	widget_class->realize = gnome_app_realize;
+	widget_class->show = gnome_app_show;
 }
 
 static void
@@ -134,7 +134,7 @@ gnome_app_init (GnomeApp *app)
 }
 
 static void
-gnome_app_realize (GtkWidget *widget)
+gnome_app_show (GtkWidget *widget)
 {
 	GnomeApp *app;
 
@@ -161,13 +161,13 @@ gnome_app_realize (GtkWidget *widget)
 	gtk_object_unref (GTK_OBJECT (app->layout));
 	app->layout = NULL;
 
-	if (GTK_WIDGET_CLASS (parent_class)->realize != NULL)
-		(* GTK_WIDGET_CLASS (parent_class)->realize) (widget);
-
 	gtk_signal_connect (GTK_OBJECT (app->dock),
 			    "layout_changed",
 			    GTK_SIGNAL_FUNC (layout_changed),
 			    (gpointer) app);
+
+	if (GTK_WIDGET_CLASS (parent_class)->show != NULL)
+		(*GTK_WIDGET_CLASS (parent_class)->show) (widget);
 }
 
 static void
@@ -375,6 +375,37 @@ gnome_app_set_statusbar (GnomeApp *app,
 	gtk_box_pack_start (GTK_BOX (app->vbox), hbox, FALSE, FALSE, 0);
 }
 
+/**
+ * gnome_app_set_statusbar_custom
+ * @app: Pointer to GNOME app object
+ * @container: container widget containing the statusbar
+ * @statusbar: Statusbar widget for main app window
+ *
+ * Description:
+ * Sets the status bar of the application window, but use @container
+ * as its container.
+ *
+ **/
+
+void
+gnome_app_set_statusbar_custom (GnomeApp *app,
+				GtkWidget *container,
+				GtkWidget *statusbar)
+{
+	GtkWidget *hbox;
+
+	g_return_if_fail(app != NULL);
+	g_return_if_fail(GNOME_IS_APP(app));
+	g_return_if_fail(container != NULL);
+	g_return_if_fail(GTK_IS_CONTAINER(container));
+	g_return_if_fail(statusbar != NULL);
+	g_return_if_fail(app->statusbar == NULL);
+
+	app->statusbar = GTK_WIDGET(statusbar);
+
+	gtk_box_pack_start (GTK_BOX (app->vbox), container, FALSE, FALSE, 0);
+}
+
 void
 gnome_app_add_toolbar (GnomeApp *app,
 		       GtkToolbar *toolbar,
