@@ -209,8 +209,6 @@ gnome_dock_item_init (GnomeDockItem *dock_item)
   dock_item->is_floating = FALSE;
   dock_item->in_drag = FALSE;
 
-  dock_item->grab_on_map_event = FALSE;
-
   dock_item->dragoff_x = 0;
   dock_item->dragoff_y = 0;
 
@@ -311,16 +309,19 @@ static gint
 gnome_dock_item_map_event (GtkWidget *widget,
                            GdkEventAny *event)
 {
+#if 0
   GnomeDockItem *dock_item;
 
   g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GNOME_IS_DOCK_ITEM (widget), FALSE);
 
   dock_item = GNOME_DOCK_ITEM (widget);
-  if (dock_item->grab_on_map_event)
+  if (dock_item->grab_on_map_event > 0)
     {
       gint x, y;
 
+      dock_item->grab_on_map_event--;
+
       if (dock_item->is_floating)
         {
           /* FIXME: This is an *ugly* hack!  */
@@ -333,6 +334,7 @@ gnome_dock_item_map_event (GtkWidget *widget,
 
       gnome_dock_item_grab_pointer (dock_item);
     }
+#endif
 
   return TRUE;
 }
@@ -491,14 +493,14 @@ gnome_dock_item_size_request (GtkWidget      *widget,
   dock_item = GNOME_DOCK_ITEM (widget);
 
   /* If our child is not visible, we still request its size, since
-     we won't have any usefull hint for our size otherwise.  */
+     we won't have any useful hint for our size otherwise.  */
   if (bin->child != NULL)
     gtk_widget_size_request (bin->child, &bin->child->requisition);
 
   if (dock_item->orientation == GTK_ORIENTATION_HORIZONTAL)
     {
       requisition->width = 
-        GNOME_DOCK_ITEM_NOT_LOCKED(dock_item) ? DRAG_HANDLE_SIZE : 0;
+        GNOME_DOCK_ITEM_NOT_LOCKED (dock_item) ? DRAG_HANDLE_SIZE : 0;
       if (bin->child != NULL)
         {
           requisition->width += bin->child->requisition.width;
@@ -510,7 +512,7 @@ gnome_dock_item_size_request (GtkWidget      *widget,
   else
     {
       requisition->height = 
-        GNOME_DOCK_ITEM_NOT_LOCKED(dock_item) ? DRAG_HANDLE_SIZE : 0;
+        GNOME_DOCK_ITEM_NOT_LOCKED (dock_item) ? DRAG_HANDLE_SIZE : 0;
       if (bin->child != NULL)
         {
           requisition->width = bin->child->requisition.width;
@@ -643,7 +645,7 @@ gnome_dock_item_paint (GtkWidget      *widget,
   GdkRectangle rect;
   gint drag_handle_size = DRAG_HANDLE_SIZE;
 
-  if (!GNOME_DOCK_ITEM_NOT_LOCKED(widget))
+  if (!GNOME_DOCK_ITEM_NOT_LOCKED (widget))
     drag_handle_size = 0;
 
   bin = GTK_BIN (widget);
@@ -687,7 +689,7 @@ gnome_dock_item_paint (GtkWidget      *widget,
   /* We currently draw the handle _above_ the relief of the dockitem.
      It could also be drawn on the same level...  */
 
-  if (GNOME_DOCK_ITEM_NOT_LOCKED(di))
+  if (GNOME_DOCK_ITEM_NOT_LOCKED (di))
     {
       
       rect.x = 0;
@@ -701,10 +703,12 @@ gnome_dock_item_paint (GtkWidget      *widget,
       else
         {
           rect.width = width;
-      rect.height = DRAG_HANDLE_SIZE;
+          rect.height = DRAG_HANDLE_SIZE;
         }
       
-      draw_textured_frame (widget, di->bin_window, &rect, GTK_SHADOW_OUT, event ? &event->area : area);
+      draw_textured_frame (widget, di->bin_window, &rect,
+                           GTK_SHADOW_OUT,
+                           event ? &event->area : area);
     }    
   
   if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
@@ -844,7 +848,6 @@ gnome_dock_item_button_changed (GtkWidget      *widget,
       gtk_grab_remove (widget);
       
       di->in_drag = FALSE;
-      di->grab_on_map_event = FALSE;
 
       gtk_signal_emit (GTK_OBJECT (widget),
                        dock_item_signals[DOCK_DRAG_END]);
@@ -875,14 +878,10 @@ gnome_dock_item_motion (GtkWidget      *widget,
   if (event->window != di->bin_window)
     return FALSE;
 
-  gdk_window_get_origin (di->float_window, &ox, &oy);
-  gdk_window_get_pointer (di->float_window, &new_x, &new_y, NULL);
-  new_x += ox;
-  new_y += oy;
-#if 0
+  gdk_window_get_pointer (NULL, &new_x, &new_y, NULL);
+  
   new_x -= di->dragoff_x;
   new_y -= di->dragoff_y;
-#endif
 
   gtk_signal_emit (GTK_OBJECT (widget),
                    dock_item_signals[DOCK_DRAG_MOTION],
@@ -977,6 +976,16 @@ gnome_dock_item_delete_event (GtkWidget *widget,
 
 
 
+/**
+ * gnome_dock_item_new:
+ * @name: Name for the new item
+ * @behavior: Behavior for the new item
+ * 
+ * Description: Create a new GnomeDockItem named @name, with the
+ * specified @behavior.
+ * 
+ * Returns: A new GnomeDockItem widget.
+ **/
 GtkWidget*
 gnome_dock_item_new (const gchar *name,
                      GnomeDockItemBehavior behavior)
@@ -991,18 +1000,42 @@ gnome_dock_item_new (const gchar *name,
   return GTK_WIDGET (new);
 }
 
+/**
+ * gnome_dock_item_get_child:
+ * @item: A GnomeDockItem widget
+ * 
+ * Description: Retrieve the child of @item.
+ * 
+ * Returns: The child of @item.
+ **/
 GtkWidget *
 gnome_dock_item_get_child (GnomeDockItem *item)
 {
   return GTK_BIN (item)->child;
 }
 
+/**
+ * gnome_dock_item_get_name:
+ * @item: A GnomeDockItem widget.
+ * 
+ * Description: Retrieve the name of @item.
+ * 
+ * Return value: The name of @item as a malloc()ed zero-terminated
+ * string.
+ **/
 gchar *
 gnome_dock_item_get_name (GnomeDockItem *item)
 {
   return g_strdup (item->name);
 }
 
+/**
+ * gnome_dock_item_set_shadow_type:
+ * @dock_item: A GnomeDockItem widget
+ * @type: The shadow type for @dock_item
+ * 
+ * Description: Set the shadow type for @dock_item.
+ **/
 void
 gnome_dock_item_set_shadow_type (GnomeDockItem  *dock_item,
                                  GtkShadowType  type)
@@ -1020,6 +1053,14 @@ gnome_dock_item_set_shadow_type (GnomeDockItem  *dock_item,
     }
 }
 
+/**
+ * gnome_dock_item_get_shadow_type:
+ * @dock_item: A GnomeDockItem widget.
+ * 
+ * Description: Retrieve the shadow type of @dock_item.
+ * 
+ * Returns: @dock_item's shadow type.
+ **/
 GtkShadowType
 gnome_dock_item_get_shadow_type (GnomeDockItem  *dock_item)
 {
@@ -1029,6 +1070,15 @@ gnome_dock_item_get_shadow_type (GnomeDockItem  *dock_item)
   return dock_item->shadow_type;
 }
 
+/**
+ * gnome_dock_item_set_orientation:
+ * @dock_item: A GnomeDockItem widget
+ * @orientation: New orientation for @dock_item
+ * 
+ * Description: Set the orientation for @dock_item.
+ * 
+ * Returns: %TRUE if the operation succeeds, %FALSE if it fails.
+ **/
 gboolean
 gnome_dock_item_set_orientation (GnomeDockItem *dock_item,
                                  GtkOrientation orientation)
@@ -1058,6 +1108,14 @@ gnome_dock_item_set_orientation (GnomeDockItem *dock_item,
   return TRUE;
 }
 
+/**
+ * gnome_dock_item_get_orientation:
+ * @dock_item: A GnomeDockItem widget.
+ * 
+ * Description: Retrieve the orientation of @dock_item.
+ * 
+ * Returns: The current orientation of @dock_item.
+ **/
 GtkOrientation
 gnome_dock_item_get_orientation (GnomeDockItem *dock_item)
 {
@@ -1069,6 +1127,14 @@ gnome_dock_item_get_orientation (GnomeDockItem *dock_item)
   return dock_item->orientation;
 }
 
+/**
+ * gnome_dock_item_get_behavior:
+ * @dock_item: A GnomeDockItem widget.
+ * 
+ * Description: Retrieve the behavior of @dock_item.
+ * 
+ * Returns: The behavior of @dock_item.
+ **/
 GnomeDockItemBehavior
 gnome_dock_item_get_behavior (GnomeDockItem *dock_item)
 {
@@ -1080,6 +1146,10 @@ gnome_dock_item_get_behavior (GnomeDockItem *dock_item)
   return dock_item->behavior;
 }
 
+
+
+/* Private interface.  */
+
 void
 gnome_dock_item_grab_pointer (GnomeDockItem *item)
 {
@@ -1087,17 +1157,15 @@ gnome_dock_item_grab_pointer (GnomeDockItem *item)
 
   fleur = gdk_cursor_new (GDK_FLEUR);
 
-  if (gdk_pointer_grab (item->bin_window,
+  /* Hm, not sure this is the right thing to do, but it seems to work.  */
+  while (gdk_pointer_grab (item->bin_window,
                         FALSE,
                         (GDK_BUTTON1_MOTION_MASK |
                          GDK_POINTER_MOTION_HINT_MASK |
                          GDK_BUTTON_RELEASE_MASK),
                         NULL,
                         fleur,
-                        GDK_CURRENT_TIME) != 0)
-    item->grab_on_map_event = TRUE; /* Delay until we are mapped.  */
-  else
-    item->grab_on_map_event = FALSE;
+                        GDK_CURRENT_TIME) != 0);
 
   gdk_cursor_destroy (fleur);
 }
