@@ -1,4 +1,3 @@
-
 /* gnome-dock.c
 
    Copyright (C) 1998 Free Software Foundation
@@ -750,9 +749,6 @@ drag_new (GnomeDock *dock,
 
   DEBUG (("entering function"));
 
-  x -= item->dragoff_x;
-  y -= item->dragoff_y;
-
   new_band = NULL;
 
   /* We need a new band next to `where', but we try to re-use the band
@@ -789,6 +785,10 @@ drag_new (GnomeDock *dock,
                                         ? GTK_ORIENTATION_VERTICAL
                                         : GTK_ORIENTATION_HORIZONTAL));
 
+      /* This is mostly to remember that `drag_allocation' for this
+         child is bogus, as it was not previously allocated.  */
+      new_band->new_for_drag = TRUE;
+
       if (where == NULL)
         *area = where = g_list_prepend (*area, new_band);
       else if (where->next == NULL)
@@ -808,9 +808,6 @@ drag_new (GnomeDock *dock,
   /* Prepare the band for dragging of `item'.  */
   gnome_dock_band_drag_begin (new_band, item);
 
-  /* Reparenting will remove the grab, so we need to redo it.  */
-  gnome_dock_item_grab_pointer (item);
-
   /* Set the offset of `item' in the band.  */
   if (is_vertical)
     gnome_dock_band_set_child_offset (new_band, GTK_WIDGET (item),
@@ -830,9 +827,6 @@ drag_to (GnomeDock *dock,
          gint x, gint y,
          gboolean is_vertical)
 {
-  x -= item->dragoff_x;
-  y -= item->dragoff_y;
-
   DEBUG (("x %d y %d", x, y));
 
   return gnome_dock_band_drag_to (GNOME_DOCK_BAND (where->data), item, x, y);
@@ -852,28 +846,25 @@ drag_floating (GnomeDock *dock,
   item_widget = GTK_WIDGET (item);
   dock_widget = GTK_WIDGET (dock);
 
-  x -= item->dragoff_x;
-  y -= item->dragoff_y;
-  rel_x -= item->dragoff_x;
-  rel_y -= item->dragoff_y;
-
   if (item_widget->parent != dock_widget)
     {
-      GtkAllocation *dock_allocation;
-      GtkAllocation *client_allocation = NULL;
+      GtkAllocation *dock_allocation, *client_allocation;
 
       /* The item is currently not floating (so it is not our child).
          Make it so if we are outside the docking areas.  */
 
       dock_allocation = &dock_widget->allocation;
-      if(dock->client_area)
+      if (dock->client_area)
         client_allocation = &dock->client_area->allocation;
+      else
+        client_allocation = NULL;
 
-      if (rel_x < dock_allocation->x
-          || rel_x >= dock_allocation->x + dock_allocation->width
-          || rel_y < dock_allocation->y
-          || rel_y >= dock_allocation->y + dock_allocation->height
-          || (client_allocation && rel_x >= client_allocation->x
+      if (rel_x < 0 
+          || rel_x >= dock_allocation->width
+          || rel_y < 0
+          || rel_y >= dock_allocation->height
+          || (client_allocation != NULL
+              && rel_x >= client_allocation->x
               && rel_x < client_allocation->x + client_allocation->width
               && rel_y >= client_allocation->y
               && rel_y < client_allocation->y + client_allocation->height))
@@ -914,64 +905,37 @@ drag_check (GnomeDock *dock,
             gint x, gint y,
             gboolean is_vertical)
 {
-#define NEW_BAND_SNAP 8
   GList *lp;
   GtkAllocation *alloc;
-  GnomeDockBand *parent_band = NULL;
-  gint alone = FALSE;
-
-  if(!item->is_floating)
-    parent_band = GNOME_DOCK_BAND(GTK_WIDGET(item)->parent);
-
-  if(parent_band)
-     alone = gnome_dock_band_get_num_children(parent_band) <= 1;
 
   for (lp = *area; lp != NULL; lp = lp->next)
     {
-      alloc = &(GTK_WIDGET(lp->data)->allocation);
+      GnomeDockBand *band;
 
-      if (x >= alloc->x && x < alloc->x + alloc->width &&
-          y >= alloc->y && y < alloc->y + alloc->height)
+      band = GNOME_DOCK_BAND (lp->data);
+
+      if (! band->new_for_drag)
         {
-          if (is_vertical)
-            {
-              if (x > alloc->x + alloc->width - NEW_BAND_SNAP)
-                {
-                  if(alone && lp->next)
-                    return drag_to (dock, item, lp->next, x, y, FALSE);
-                  else if(!lp->next)
-                    return drag_new (dock, item, area, lp, x, y, FALSE);
-                  else if(parent_band == GNOME_DOCK_BAND(lp->data))
-                    return drag_to (dock, item, lp, x, y, FALSE);
-                }
-              else
-                {
-                  if(!alone && x < alloc->x + NEW_BAND_SNAP)
-                    return drag_new (dock, item, area, lp->prev, x, y, FALSE);
-                  else if (x > alloc->x + NEW_BAND_SNAP ||
-                           parent_band == GNOME_DOCK_BAND(lp->data))
-                    return drag_to (dock, item, lp, x, y, FALSE);
-                }
-            }
-          else
-            {
-              if (y > alloc->y + alloc->height - NEW_BAND_SNAP)
-                {
-                  if(alone && lp->next)
-                    return drag_to (dock, item, lp->next, x, y, FALSE);
-                  else if(!lp->next)
-                    return drag_new (dock, item, area, lp, x, y, FALSE);
-                  else if(parent_band == GNOME_DOCK_BAND(lp->data))
-                    return drag_to (dock, item, lp, x, y, FALSE);
+          alloc = &band->drag_allocation;
 
+          if (x >= alloc->x && x < alloc->x + alloc->width
+              && y >= alloc->y && y < alloc->y + alloc->height)
+            {
+              if (is_vertical)
+                {
+                  if (x < alloc->x + alloc->width / 2
+                      && drag_to (dock, item, lp, x, y, TRUE))
+                    return TRUE;
+                  else
+                    return drag_new (dock, item, area, lp, x, y, TRUE);
                 }
               else
                 {
-                  if(!alone && y < alloc->y + NEW_BAND_SNAP)
-                    return drag_new (dock, item, area, lp->prev, x, y, FALSE);
-                  else if (y > alloc->y + NEW_BAND_SNAP ||
-                           parent_band == GNOME_DOCK_BAND(lp->data))
-                    return drag_to (dock, item, lp, x, y, FALSE);
+                  if (y < alloc->y + alloc->height / 2
+                      && drag_to (dock, item, lp, x, y, FALSE))
+                    return TRUE;
+                  else 
+                    return drag_new (dock, item, area, lp, x, y, FALSE);
                 }
             }
         }
@@ -996,10 +960,10 @@ drag_begin (GtkWidget *widget, gpointer data)
 
   /* Communicate all the bands that `widget' is currently being
      dragged.  */
-  g_list_foreach (dock->top_bands, (GFunc)gnome_dock_band_drag_begin, item);
-  g_list_foreach (dock->bottom_bands, (GFunc)gnome_dock_band_drag_begin, item);
-  g_list_foreach (dock->right_bands, (GFunc)gnome_dock_band_drag_begin, item);
-  g_list_foreach (dock->left_bands, (GFunc)gnome_dock_band_drag_begin, item);
+  g_list_foreach (dock->top_bands, (GFunc) gnome_dock_band_drag_begin, item);
+  g_list_foreach (dock->bottom_bands, (GFunc) gnome_dock_band_drag_begin, item);
+  g_list_foreach (dock->right_bands, (GFunc) gnome_dock_band_drag_begin, item);
+  g_list_foreach (dock->left_bands, (GFunc) gnome_dock_band_drag_begin, item);
 }
 
 
@@ -1142,6 +1106,12 @@ drag_motion (GtkWidget *widget,
   if (dock->floating_items_allowed
       && ! (item_behavior & GNOME_DOCK_ITEM_BEH_NEVER_DETACH))
     drag_floating (dock, item, x, y, rel_x, rel_y);
+
+  /* If still not floating, fall back to moving the item in its own
+     band.  */
+  if (! item->is_floating)
+    gnome_dock_band_drag_to (GNOME_DOCK_BAND (GTK_WIDGET (item)->parent),
+                             item, rel_x, rel_y);
 }
 
 
@@ -1268,6 +1238,13 @@ gnome_dock_get_type (void)
   return dock_type;
 }
 
+/**
+ * gnome_dock_new:
+ * 
+ * Description: Creates a new GnomeDock widget.
+ * 
+ * Return value: The new widget.
+ **/
 GtkWidget *
 gnome_dock_new (void)
 {
@@ -1285,6 +1262,14 @@ gnome_dock_new (void)
   return widget;
 }
 
+/**
+ * gnome_dock_allow_floating_items:
+ * @dock: A pointer to a GnomeDock widget
+ * @enable: Specifies whether floating items are allowed in this dock
+ * 
+ * Description: Enable or disable floating items on @dock, according
+ * to @enable.
+ **/
 void
 gnome_dock_allow_floating_items (GnomeDock *dock,
                                  gboolean enable)
@@ -1292,6 +1277,23 @@ gnome_dock_allow_floating_items (GnomeDock *dock,
   dock->floating_items_allowed = enable;
 }
 
+/**
+ * gnome_dock_add_item:
+ * @dock: A pointer to a GnomeDock widget
+ * @item: The item to add
+ * @placement: Placement for the new item
+ * @band_num: Number of the band the new item must be added to
+ * @position: Position of the item in the specified band
+ * @offset: Offset (in pixels) from the previous item in the same band
+ * @in_new_band: Specifies whether a new band must be created for this item
+ * 
+ * Description: Add @item to @dock.  @placement can be either
+ * %GNOME_DOCK_TOP, %GNOME_DOCK_RIGHT, %GNOME_DOCK_BOTTOM or
+ * %GNOME_DOCK_LEFT, and specifies what area of the dock should
+ * contain the item.  If @in_new_band is %TRUE, a new dock band is
+ * created at the position specified by @band_num; otherwise, the item
+ * is added to the @band_num'th band.
+ **/
 void
 gnome_dock_add_item (GnomeDock *dock,
                      GnomeDockItem *item,
@@ -1372,6 +1374,18 @@ gnome_dock_add_item (GnomeDock *dock,
   gtk_signal_emit (GTK_OBJECT (dock), dock_signals[LAYOUT_CHANGED]);
 }
 
+/**
+ * gnome_dock_add_floating_item:
+ * @dock: A GnomeDock widget
+ * @item: The item to be added
+ * @x: X-coordinate for the floating item
+ * @y: Y-coordinate for the floating item
+ * @orientation: Orientation for the new item.
+ * 
+ * Description: Add @item to @dock and make it floating at the
+ * specified (@x, @y) coordinates (relative to the root window of the
+ * screen).
+ **/
 void
 gnome_dock_add_floating_item (GnomeDock *dock,
                               GnomeDockItem *item,
@@ -1401,6 +1415,13 @@ gnome_dock_add_floating_item (GnomeDock *dock,
   gtk_signal_emit (GTK_OBJECT (dock), dock_signals[LAYOUT_CHANGED]);
 }
 
+/**
+ * gnome_dock_set_client_area:
+ * @dock: A GnomeDock widget
+ * @widget: The widget to be used for the client area.
+ * 
+ * Description: Specify a widget for the dock's client area.
+ **/
 void
 gnome_dock_set_client_area (GnomeDock *dock, GtkWidget *widget)
 {
@@ -1442,12 +1463,41 @@ gnome_dock_set_client_area (GnomeDock *dock, GtkWidget *widget)
     gtk_widget_unref (widget);
 }
 
+/**
+ * gnome_dock_get_client_area:
+ * @dock: A GnomeDock widget.
+ * 
+ * Description: Retrieve the widget being used as the client area in
+ * @dock.
+ * 
+ * Returns: The client area widget.
+ **/
 GtkWidget *
 gnome_dock_get_client_area (GnomeDock *dock)
 {
   return dock->client_area;
 }
 
+/**
+ * gnome_dock_get_item_by_name:
+ * @dock: A GnomeDock widget.
+ * @name: The name of the dock item to retrieve
+ * @placement_return: A pointer to a variable holding the item's placement
+ * @num_band_return: A pointer to a variable holding the band number
+ * @band_position_return: A pointer to a variable holding the position
+ * of the item within the band
+ * @offset_return: A pointer to a variable holding the offset of the item
+ * from the previous item in the same band
+ * 
+ * Description: Retrieve the dock item named @name; information about
+ * its position in the dock is returned via @placement_return,
+ * @num_band_return, @band_position_return and @offset_return.  If
+ * the placement is %GNOME_DOCK_FLOATING *@num_band_return,
+ * *@band_position_return and *@offset_return are not set.
+ * 
+ * Returns: The named GnomeDockItem widget, or %NULL if no item with
+ * such name exists.
+ **/
 GnomeDockItem *
 gnome_dock_get_item_by_name (GnomeDock *dock,
                              const gchar *name,
@@ -1525,6 +1575,14 @@ layout_add_bands (GnomeDock *dock,
     }
 }
 
+/**
+ * gnome_dock_get_layout:
+ * @dock: A GnomeDock widget
+ * 
+ * Description: Retrieve the layout of @dock.
+ * 
+ * Returns: @dock's layout as a GnomeDockLayout object.
+ **/
 GnomeDockLayout *
 gnome_dock_get_layout (GnomeDock *dock)
 {
