@@ -59,7 +59,14 @@ struct _vtx
   /* selection stuff */
   char *selection_data;		/* actual selection */
   int selection_size;
-  
+
+  /* 256 bits of word class characters (assumes a char is 8 bits or more) */
+  unsigned char wordclass[32];
+
+  /* rendering buffer, for building output strings */
+  char *runbuffer;
+  int runbuffer_size;
+
   /* true if something selected */
   int selected;
 
@@ -83,6 +90,7 @@ void vt_draw_selection   (struct _vtx *vx);
 void vt_update_rect      (struct _vtx *vx, int sx, int sy, int ex, int ey);
 void vt_update           (struct _vtx *vt, int state);
 void vt_draw_cursor      (struct _vtx *vx, int state);
+void vt_set_wordclass    (struct _vtx *vx, unsigned char *s);
 			 
 struct _vtx *vtx_new     (int width, int height, void *user_data);
 void vtx_destroy         (struct _vtx *vx);
